"""
A mesh obj reader
"""

__author__ = 'Davide Pellis'

import numpy as np

# -----------------------------------------------------------------------------

from geolab.utilities.stringutilities import make_filepath

from geolab.mesh.globalconnectivity import faces_list

# -----------------------------------------------------------------------------


def read_mesh(file_name):
    """Read an OBJ mesh file.

    Parameters
    ----------
    file_name : str
        The path of the OBJ file to open.

    Returns
    -------
    v : np.array (V,3)
        The array of vertices  [[x_0, y_0, z_0], ..., [x_V, y_V, z_V]]
    f : list
        The list of faces [[v_i, v_j, ...], ...]

    Notes
    -----
    Files must be written without line wrap (in many CAD software,
    line wrap can be disabled in the OBJ save options).
    """
    file_name = str(file_name)
    obj_file = open(file_name, encoding='utf-8')
    vertices_list = []
    uv_list = []
    f = []
    for line in obj_file:
        split_line = line.split(' ')
        if split_line[0] == 'v':
            split_x = split_line[1].split('\n')
            x = float(split_x[0])
            split_y = split_line[2].split('\n')
            y = float(split_y[0])
            split_z = split_line[3].split('\n')
            try:
                z = float(split_z[0])
            except ValueError:
                print('WARNING: disable line wrap when saving .obj')
            vertices_list.append([x, y, z])
        elif split_line[0] == 'f':
            v_list = []
            L = len(split_line)
            try:
                for i in range(1, L):
                    split_face_data = split_line[i].split('/')
                    v_list.append(int(split_face_data[0]) - 1)
                f.append(v_list)
            except ValueError:
                v_list = []
                for i in range(1, L - 1):
                    v_list.append(int(split_line[i]) - 1)
                f.append(v_list)
        if split_line[0] == 'vt':
            split_u = split_line[1].split('\n')
            u = float(split_u[0])
            split_v = split_line[2].split('\n')
            v = float(split_v[0])
            uv_list.append([u, v])
    v = np.array(vertices_list)
    try:
        f = np.array(f)
    except TypeError:
        pass
    return v, f


def save_mesh(vertices, faces=None, halfedges=None, file_name='mesh',
              overwrite=False):
    """Save the mesh as OBJ file.

    Parameters
    ----------
    vertices : np.array (H, 3)
        The array of vertices.

    Optional Parameters
    -------------------
    faces : np.array (F, n) or list of lists
        The array or list of faces.
    halfedges : np.array (H, 6)
        The array of halfedges.
    file_name : str
        The path of the OBJ file to be created, without extension.
    overwrite : bool
        If `False` (default), when the path already exists, a sequential
        number is added to file_name. If `True`, existing paths are
        overwritten.

    Returns
    -------
    str
        The path of the saves OBJ file (without extension).
    """
    path = make_filepath(file_name, 'obj', overwrite)
    obj = open(path, 'w')
    line = 'o {}\n'.format(file_name)
    obj.write(line)
    if faces is None:
        if halfedges is not None:
            faces = faces_list(halfedges)
        else:
            faces = [[]]
    for v in range(len(vertices)):
        vi = vertices[v]
        line = 'v {} {} {}\n'.format(vi[0], vi[1], vi[2])
        obj.write(line)
    for f in range(len(faces)):
        obj.write('f ')
        N = len(faces[f])
        for v in range(N - 1):
            vf = str(faces[f][v] + 1)
            obj.write(vf + ' ')
        vf = str(faces[f][N - 1] + 1)
        obj.write(vf + '\n')
    obj.close()
    out = 'Obj saved in {}'.format(path)
    print(out)
    return path.split('.')[0]

